/* -*- c++ -*- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <list>
#include <iostream>
#include <string>

#include <locale.h>


#define USE_WCHAR 1

#ifdef USE_WCHAR
typedef std::wstring String;
typedef wchar_t Char;
#else
typedef std::string String;
typedef char Char;
#endif

enum TextTokenState {
	TEXT = 0,
	TAG = 1,
};

class Font
{
public:
	int strWidth(char* text) { return strlen(text); };
};

class TextStyle
{
public:
	Font* font;
	int color;
	bool active;
	long uri;
};


class TextSpan
{
public:
	std::string text;
	int x, y;
	int width, height;
	int line;
	int open;
	TextSpan* link_to_previous_for_multiline;
	// lists of opening and closing tags
	TextStyle openingStyle;
	TextStyle closingStyle;

	TextSpan() : text("") {};
};


class Text
{
	std::list<TextSpan*> spans;
public:
	Text() : tokenType(TEXT) {};
	~Text() {};
	void parseText(char* text);
	void removeLine () {};
	void print();
protected:
	TextTokenState state;

	bool escaped;
	String buffer;
	TextTokenState tokenType;
	String token;
	TextSpan* getLastOpen();
	String* getToken();
};

String* Text::getToken()
{
	for (String::iterator it = buffer.begin(); it != buffer.end(); it++) {
		//std::cout << token << ' ' << tokenType << ' ' << *it << std::endl;
		if (escaped) {
			token += *it;
			escaped = false;
		} else if (*it == '\\') {
			escaped = true;
		} else if (token.empty()) {
			tokenType = (*it == '[') ? TAG : TEXT;
			token += *it;
		} else if (tokenType == TEXT && *it == '[') {
			buffer.erase(buffer.begin(), it);
			return &token;
		} else if ((tokenType == TAG && *it == ']') || (tokenType == TEXT && *it == '\n')) {
			token += *it++;
			buffer.erase(buffer.begin(), it);
			return &token;
		} else {
			token += *it;
		}
	}

	buffer.clear();
	if (tokenType == TAG || token.empty())
		return NULL;
	else
		return &token;
}


void Text::parseText(char* text)
{
#ifdef USE_WCHAR
	wchar_t mbtext[1000];
	mbstowcs (mbtext, text, 1000);
	buffer += mbtext;
#else
	buffer += text;
#endif

	String *t = getToken ();
	while (t) {
		printf ("TTT %ls\n", t->c_str());
		//std::cout << "TTT " << *t << "\n";
		t->clear();
		t = getToken ();
	}
//
//	printf("spans len %d\n", spans.size());
//	char *start = text;
//
//	//if (!spans.size())
//	//	spans.push_back(new TextSpan());
//
//	//TextSpan *span = spans.back();
//
//	last = spans.back();
//	lastOpen = getLastOpen();
//
//	if (state == 0 && tag) {
//		// process the tag
//	} else if (state == 0 && *text == '[') {
//		state = 1;
//		tag = readToken();
//		// ...
//	} else if (state == 0) {
//		span.text += readToken();
//	} else {
//		tag += readToken();
//	}
//
//	while (*text) {
//		printf("start: %d; span: %p; text: %s\n", text-start, span, text);
//
//		if (*text == '[') {
//			printf("C[: %d span: %p\n", text-start, span);
//			// end the current span
//
//			span = new TextSpan(*span);
//			span->text = std::string();
//
//			printf("t1: %d span: %p\n", text-start, span);
//			spans.push_back(span);
//
//			text++;
//			while (*text && *text != ']') {
//				printf("  t0: %d ; text: %s\n", text-start, text);
//
//				if (strncmp("s=", text, 2) == 0) {
//					span->active = true;
//					span->uri = strtol(text+2, &text, 0);
//					//pushTag(span);
//				}
//				else if (strncmp("/s", text, 2) == 0) {
//					span->active = false;
//					text += 2;
//					//popTag(span);
//				}
//				// check that *endptr == ']'
//			}
//			if (*text == ']')
//				text++;
//		}
//
//		if (*text == '\n') {
//			printf("C\\n: %d span: %p\n", text-start, span);
//			// end the current span
//
//			span = new TextSpan(*span);
//			span->text = std::string();
//			span->line++;
//
//			printf("t1: %d span: %p\n", text-start, span);
//			spans.push_back(span);
//
//			text++;
//		}
//
//		int len = strcspn(text, "[\n");
//		printf("spans len2 %d\n", len);
//		if (len) {
//			char *tmp = (char*)malloc(len + 1);
//			memcpy(tmp, text, len);
//			tmp[len] = '\0';
//
//			span->text += std::string(tmp);
//			text += len;
//		}
//	}
}

TextSpan* Text::getLastOpen ()
{
	for (std::list<TextSpan*>::iterator it = spans.end(), b=spans.begin(); it != b; --it) {
		if ((*it)->open) {
			return *it;
		}
	}
	return NULL;
}

void Text::print()
{
	int lastLine = 0;
	for (std::list<TextSpan*>::iterator it = spans.begin(), e=spans.end(); it != e; it++) {
		if ((*it)->line != lastLine) {
			printf("\n");
			lastLine =(*it)->line;
		}
		printf("< %s >", (*it)->text.c_str());
	}
	printf("\n");
}

int main (int argc, char** argv)
{
	setlocale (LC_CTYPE, "");
	char* text = "\"By every god and his mother... well, chief, more than likely your memories just took a dive into your brain-box. With any luck, they'll come up for air \
soon, trust me. Musta been some night you had. Better hope you didn't hurt anybody or mix it up with the law... say, speaking of which, are you one of those  \
Sensates into self-mutilation or did somebody give you those scars?\"\n\
\n\
1 - [s=1]\"Could it be that I am dead? And that's why I'm here?\"[/s]\n\
2 - [s=2]\"How bad do I look?\"[/s]\n\
3 - [s=3]\"Nevermind. I had some other questions...\"[/s]\n\
4 - [s=4]Ignore Morte and explore the room.[/s]\n";

	char* text2 = "alpha beta [c]gamma[/c] delta\nžepsilon";

	Text p;

	p.parseText(text2);
	//p.print();
}

